/////////////////////////////////////////////////////////////
//                                                         //
// Copyright (c) 2003-2011 by The University of Queensland //
// Earth Systems Science Computational Centre (ESSCC)      //
// http://www.uq.edu.au/esscc                              //
//                                                         //
// Primary Business: Brisbane, Queensland, Australia       //
// Licensed under the Open Software License version 3.0    //
// http://www.opensource.org/licenses/osl-3.0.php          //
//                                                         //
/////////////////////////////////////////////////////////////


#ifndef CHECKPOINTPARAMS_H
#define CHECKPOINTPARAMS_H

#include <mpi.h>

#include <string>
#include <iostream>


class AMPIBuffer;

/**
 * Parameters associated with check-point file creation.
 */
class CheckPointParams
{
public:
  CheckPointParams(const std::string &fileNamePrefix, int time, int rank);

  ~CheckPointParams();

  /**
   * Returns full check-point file (prefix plus time and rank identifier).
   */
  std::string getFileName() const;

  void packInto(AMPIBuffer *mpiBuff) const;
  static CheckPointParams unpackFrom(AMPIBuffer *mpiBuff, MPI_Comm mpiComm=MPI_COMM_WORLD);

  //friend std::ostream& operator<<(std::ostream &oStream, const CheckPointParams &checkPointParams);

protected:
  CheckPointParams();

private:
  /**
   * Prefix of check-point file names.
   */
  std::string m_fileNamePrefix;
  
  /**
   * The simulation time at which checkpoint is created.
   */
  int m_time;
  
  /**
   * Rank of mpi process.
   */
  int m_rank;
};

#endif
